local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("SliceCommander")

function SliceCommander:ChangeVolume(val)
	if SC.SET.soundChannel == "Master" then
		SetCVar("Sound_MasterVolume",val)
	elseif SC.SET.soundChannel == "SFX" then
		SetCVar("Sound_SFXVolume",val)
	elseif SC.SET.soundChannel == "Ambience" then
		SetCVar("Sound_AmbienceVolume",val)
	elseif SC.SET.soundChannel == "Music" then
		SetCVar("Sound_MusicVolume",val)
	end
end

function SliceCommander:CreateCore()
	SliceCommander:CreateMoveFrame()
	SliceCommander:CreateEnergyFrame()
	SliceCommander:CreateComboText()
	SliceCommander:CreateSpark()
end

function SliceCommander:MoveStart(frame)
	if SC.SET.IsLocked == false then
		frame:StartMoving()
	end
end

function SliceCommander:MoveStop(frame)
	if SC.SET.IsLocked == false then
		frame:StopMovingOrSizing()
		frame:SetUserPlaced(false)
		SC.OTHER[frame:GetName()]['position'].point,
		SC.OTHER[frame:GetName()]['position'].relativeTo,
		SC.OTHER[frame:GetName()]['position'].relativePoint,
		SC.OTHER[frame:GetName()]['position'].xOfs,
		SC.OTHER[frame:GetName()]['position'].yOfs = frame:GetPoint();
	end
end

function SliceCommander:ApplySkin(isApply)
	if isApply then
		for i,v in pairs(SC.BARS) do
			if i ~= 'CP' then
				if i ~= 'THREAT' and i ~= 'HEALTH' then
					v['obj'].icon:ClearAllPoints()
					v['obj'].icon:SCPoint("RIGHT", v['obj'], "LEFT", -7, 0)
				end
				if i == 'RUP' then
					v['dot']['obj']:ClearAllPoints()
					v['dot']['obj']:SCPoint("LEFT", v['obj'], "RIGHT", 7, 0)
				end
			end
		end
	else
		for i,v in pairs(SC.BARS) do
			if i ~= 'CP' then
				if i ~= 'THREAT' and i ~= 'HEALTH' then
					v['obj'].icon:ClearAllPoints()
					v['obj'].icon:SCPoint("RIGHT", v['obj'], "LEFT", -1, 0)
				end
				if i == 'RUP' then
					v['dot']['obj']:ClearAllPoints()
					v['dot']['obj']:SCPoint("LEFT", v['obj'], "RIGHT", 1, 0)
				end
			end
		end
	end

	SliceCommander:SetTimersHeight(SC.SET.TimerHeight)
	SliceCommander:SetWidth(SC.SET.Width)
	SliceCommander:SetEnergyHeight(SC.SET.EnergyHeight)
	SliceCommander:SetIconSize(SC.SET.IconSize)
end

function SliceCommander:GetVolume()
	if SC.SET.soundChannel == "Master" then
		return GetCVar("Sound_MasterVolume")
	elseif SC.SET.soundChannel == "SFX" then
		return GetCVar("Sound_SFXVolume")
	elseif SC.SET.soundChannel == "Ambience" then
		return GetCVar("Sound_AmbienceVolume")
	elseif SC.SET.soundChannel == "Music" then
		return GetCVar("Sound_MusicVolume")
	end
end

function SliceCommander:OrderTable(tableName)
	local i = 0
	for k, v in pairs(SC_OptionsTable[tableName].args) do
		SC_OptionsTable[tableName].args[k]["order"] = i
		i = i+1
	end
	
	return SC_OptionsTable[tableName]
end

function SliceCommander:InitFont()
	SC.BarFont1 = SliceCommander:NewFont("VTimerFont2", 8, 0.7, 213/255, 200/255, 184/255)
	SC.BarFont2 = SliceCommander:NewFont("VTimerFont4", 8, 1, 213/255, 200/255, 184/255)
	SC.FontIcon = SliceCommander:NewFont("IconFont", 15, 0.5, 1, 1, 1)
end

function SliceCommander:InitIcon()
	SC.OTHER['POISON']['obj'] = SliceCommander:NewIcon("Interface\\Icons\\ability_rogue_dualweild", 'POISON')--Create poison icon warning when it left 5minute or less on the current timer
	SC.OTHER['POISON']['obj']:Show()
	SC.OTHER['POISON']['obj'].button = CreateFrame("Button", nil, UIParent, "SecureActionButtonTemplate")--Create button to reaply poison
	SC.OTHER['POISON']['obj'].button:SetFrameStrata("HIGH")
	SC.OTHER['POISON']['obj'].button:SetFrameLevel(5)
	SC.OTHER['POISON']['obj'].button:SetPoint("BOTTOMLEFT", SC.OTHER['POISON']['obj'], "BOTTOMLEFT")
	SC.OTHER['POISON']['obj'].button:SetPoint("TOPRIGHT", SC.OTHER['POISON']['obj'], "TOPRIGHT")
	SC.OTHER['POISON']['obj'].button:RegisterForClicks("LeftButtonUp", "RightButtonUp")
	SC.OTHER['POISON']['obj'].button:SetAttribute("unit", "player")
	SC.OTHER['POISON']['obj'].button:SetAttribute("*type1", "spell")
	SC.OTHER['POISON']['obj'].button:SetAttribute("*type2", "spell")
	SC.OTHER['POISON']['obj'].button:EnableMouse(false)
	SC.OTHER['BF']['obj'] = SliceCommander:NewIcon("Interface\\Icons\\Ability_Warrior_PunishingBlow", 'BF')--Create Blade fury icon display when BF is enable
	SC.OTHER['SOTD']['obj'] = SliceCommander:NewIcon("Interface\\Icons\\inv_misc_volatileshadow", 'SOTD')--Create Shadow of the destroyer icon display when stack rise 30
	SC.OTHER['SOTD']['obj'].overlay:Show()
end

function SliceCommander:InitGCDBar()
	SC.OTHER['AS']['obj'] = SliceCommander:ThinBar("BOTTOMLEFT", "TOPLEFT", 0 , 0)--Create Autoshot
	SC.OTHER['AS']['obj']:SetStatusBarColor(180/255, 38/255, 38/255)
end

function SliceCommander:DisableIcon(obj, val)
	for i,v in pairs(obj.iconBorder) do
		v:Hide()
	end
	obj:SetMinMaxValues(0, val)--Threat value should be between 0 and val
end

function SliceCommander:InitBar()
	SC.BARS['CP']['obj'] = SliceCommander:CPFrame()--create Combo Point bar
	local maxCP = UnitPowerMax("player", SPELL_POWER_COMBO_POINTS)

	for i = 1, maxCP do
		SC.BARS['CP']['obj'].combos[i].bg:SetVertexColor(unpack(SC.SET.CP[maxCP][i]))
	  	SC.BARS['CP']['obj'].combos[i].border[2]:SetBackdropColor(unpack(SC.V.wColor))
	end
	
	for i,v in pairs(SC.BARS) do
		if i ~= "CP" then
			SC.BARS[i].obj = SliceCommander:NewFrame(v.color)
		end
	end

	--GLOBAL
	SC.BARS['KSCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_kidneyshot")
	SC.BARS['TOT']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_TricksOftheTrade")
	SC.BARS['VAN']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Vanish")
	SC.BARS['SS']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_Shadowstep")
	SC.BARS['MD']['obj'].icon:SetTexture("Interface\\Icons\\achievement_bg_killingblow_berserker")
	SC.BARS['DFA']['obj'].icon:SetTexture("Interface\\Icons\\spell_rogue_deathfromabove")
	SC.BARS['CVCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_crimsonvial")

	--ASSASSINATION
	SC.BARS['VENCD']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_Deadliness")
	SC.BARS['EX']['obj'].icon:SetTexture("Interface\\Icons\\ability_deathwing_bloodcorruption_earth")
	SC.BARS['GARCD']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_garrote")
	SC.BARS['KBCD']['obj'].icon:SetTexture("Interface\\Icons\\inv_knife_1h_artifactgarona_d_01")

	--OUTLAW
	SC.BARS['ARCD']['obj'].icon:SetTexture("Interface\\Icons\\Spell_Shadow_ShadowWordDominate")
	SC.BARS['KSP']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_Murderspree")
	SC.BARS['GH']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_grapplinghook")
	SC.BARS['CB']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_cannonballbarrage")
	SC.BARS['PA']['obj'].icon:SetTexture("Interface\\Icons\\achievement_character_human_male")
	SC.BARS['BE']['obj'].icon:SetTexture("Interface\\Icons\\inv_weapon_rifle_01")
	SC.BARS['RB']['obj'].icon:SetTexture("Interface\\Icons\\ability_rogue_rollthebones")

	--SUBTLETY
	SC.BARS['SDCD']['obj'].icon:SetTexture("Interface\\Icons\\Ability_Rogue_Shadowdance")
	SC.BARS['SBCD']['obj'].icon:SetTexture("Interface\\Icons\\inv_knife_1h_grimbatolraid_d_03")
	SC.BARS['GB']['obj'].icon:SetTexture("Interface\\Icons\\inv_knife_1h_artifactfangs_d_01")

	SliceCommander:DisableIcon(SC.BARS['THREAT']['obj'], 110)
	SliceCommander:DisableIcon(SC.BARS['HEALTH']['obj'], 100)
	SliceCommander:DisableIcon(SC.BARS['THEALTH']['obj'], 100)

	SliceCommander:NewDotPowerIcon(SC.BARS['RUP'])
end

function SliceCommander:InitConstantValue()
	local height, width
	height = tonumber(string.match(GetCVar("gxWindowedResolution"), "%d+x(%d+)"))
	width = tonumber(string.match(GetCVar("gxWindowedResolution"), "(%d+)x+%d"))
	width = width*SC.SET.eyefinity--eyefinity is wired
	--SC.LastEnergy = UnitMana('player')
	SC.curCombo = 0
	SC.RUP_AlertPending = 3
	SC.SND_AlertPending = 3
	SC.V = {
		['bColor'] = {.1,.1,.1},
		['wColor'] = {.6,.6,.6},
		['pxH'] = UIParent:GetHeight() / height,
		['pxW'] = UIParent:GetWidth() / width,
	}

	SC.Spark = {}
end

function SliceCommander:ShowHideBorder(visible)
	if visible then
		for i,v in pairs(VTimerEnergy.border) do
			v:Show()
		end
		for i,v in pairs(SC.BARS) do
			if i ~= "CP" then
				for j,w in pairs(v['obj'].border) do
					w:Show()
				end
				if i ~= "THREAT" and i ~= "HEALTH" and i ~= "THEALTH" then
					for j,w in pairs(v['obj'].iconBorder) do
						w:Show()
					end
				end
			end
		end
		for i,v in pairs(SC.BARS['RUP']['dot']['obj'].border) do
			v:Show()
		end
		for i,v in pairs(SC.BARS['CP']['obj'].combos) do
			for j,w in pairs(v.border) do
				w:Show()
			end
		end
	else
		for i,v in pairs(VTimerEnergy.border) do
			v:Hide()
		end
		for i,v in pairs(SC.BARS) do
			if i ~= "CP" then
				for j,w in pairs(v['obj'].border) do
					w:Hide()
				end
				if i ~= "THREAT" and i ~= "CP" and i ~= "HEALTH" and i ~= "THEALTH" then
					for j,w in pairs(v['obj'].iconBorder) do
						w:Hide()
					end
				end
			end
		end
		for i,v in pairs(SC.BARS['CP']['obj'].combos) do
			for j,w in pairs(v.border) do
				w:Hide()
			end
		end
		for i,v in pairs(SC.BARS['RUP']['dot']['obj'].border) do
			v:Hide()
		end
	end
end

function SliceCommander:ApplyConf()
	SliceCommander:LockCkecked(SC.SET.IsLocked)
	SliceCommander:SetTimersHeight(SC.SET.TimerHeight)
	SliceCommander:SetWidth(SC.SET.Width)
	SliceCommander:SetEnergyHeight(SC.SET.EnergyHeight)
	SliceCommander:SetIconSize(SC.SET.IconSize)
	SliceCommander:Config_RetextureBars(SliceCommander:BarTexture())
	SliceCommander:ToggleRupPower(SC.SET.power.RUP)
	SliceCommander:FrameOpacity(SC.SET.FrameOpacity)
	SliceCommander:SetTransparency(SC.SET.EnergyTrans)
	SliceCommander:ApplySkin(SC.SET.skin)
	SliceCommander:ShowHideBorder(SC.SET.skin)
	SliceCommander:ACTIVE_TALENT_GROUP_CHANGED()
end

function SliceCommander:ToggleRupPower(toggle)
	if toggle and not SC.BARS['RUP']['dot']['obj']:IsVisible() then
		SC.BARS['RUP']['dot']['obj']:Show()
	end
	if not toggle and SC.BARS['RUP']['dot']['obj']:IsVisible() then
		SC.BARS['RUP']['dot']['obj']:Hide()
	end
	SliceCommander:SetWidth(SC.SET.Width)
end

function SliceCommander:ConfigSoundMenuInit(WhichMenu)
	local returnArray = { ["None"] = L["None"] }
	for ignore, SoundName in pairs(SliceCommander_SoundMenu[WhichMenu]) do
		returnArray[SoundName] = SoundName
	end
	return returnArray
end

function SliceCommander:SetSparks()
	local spec = GetSpecialization()
	
	SC.Spark[1]:SCPoint("TOPLEFT", VTimerEnergy, "TOPLEFT", ((SC.SET.Energy[spec][1] / UnitManaMax("player") * SC.SET.Width)-1), 0)
	SC.Spark[2]:SCPoint("TOPLEFT", VTimerEnergy, "TOPLEFT", ((SC.SET.Energy[spec][2] / UnitManaMax("player") * SC.SET.Width)-1), 0)
	SC.Spark[3]:SCPoint("TOPLEFT", VTimerEnergy, "TOPLEFT", ((SC.SET.Energy[spec][3] / UnitManaMax("player") * SC.SET.Width)-1), 0)
end

function SliceCommander:LockCkecked(checked)
	if checked then
		SliceCommander:SetTransparency(SC.SET.FrameOpacity)
		SliceCommanderFrame:SetAlpha(0)
		SliceCommanderFrame:EnableMouse(false)
		SC.OTHER['POISON']['obj']['moveframe']:SetAlpha(0)
		SC.OTHER['POISON']['obj']['moveframe']:EnableMouse(false)
		SC.OTHER['BF']['obj']['moveframe']:SetAlpha(0)
		SC.OTHER['BF']['obj']['moveframe']:EnableMouse(false)
		SC.OTHER['SOTD']['obj']['moveframe']:SetAlpha(0)
		SC.OTHER['SOTD']['obj']['moveframe']:EnableMouse(false)
		SC.SET.IsLocked = true
	else
		SliceCommander:SetTransparency(100)
		SliceCommanderFrame:SetAlpha(.5)
		SliceCommanderFrame:EnableMouse(true)
		SC.OTHER['POISON']['obj']['moveframe']:SetAlpha(.5)
		SC.OTHER['POISON']['obj']['moveframe']:EnableMouse(true)
		SC.OTHER['POISON']['obj']:SetAlpha(1)
		SC.OTHER['POISON']['obj']:Show()
		SC.OTHER['BF']['obj']['moveframe']:SetAlpha(.5)
		SC.OTHER['BF']['obj']['moveframe']:EnableMouse(true)
		SC.OTHER['BF']['obj']:Show()
		SC.OTHER['SOTD']['obj']['moveframe']:SetAlpha(.5)
		SC.OTHER['SOTD']['obj']['moveframe']:EnableMouse(true)
		SC.OTHER['SOTD']['obj']:Show()
		SC.SET.IsLocked = false
	end
end

function SliceCommander:Config_RetextureBars(texture)
	VTimerEnergy:SetStatusBarTexture(texture)
	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil and i ~= 'CP' then
			v['obj']:SetStatusBarTexture(texture)
		end
	end

	for i = 1, 8 do
		SC.BARS['CP']['obj'].combos[i].bg:SetTexture(texture)
	end
end

function SliceCommander:SetWidth(w)
	local pxMargin = 1
	local maxCP = UnitPowerMax("player", SPELL_POWER_COMBO_POINTS)
	if SC.SET.skin then
		pxMargin = 7
	end
	local widthCP = math.floor((w-(pxMargin*(maxCP-1)))/maxCP)
	w = widthCP*maxCP+pxMargin*(maxCP-1)

	local h = SC.SET.TimerHeight

	if SC.SET.skin then
		--3 pixel border x2 + 1 pixel margin
		h = h + 6 + 1
	end
	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil and i ~= 'THREAT' and i ~= 'CP' and i ~= "HEALTH" and i ~= "THEALTH" then
			if i == 'RUP' and SC.SET.power.RUP then
				SliceCommander:SetSizeBorder(w-(h*2), 0, v['obj'].border)
				v['obj']:SCWidth(w-(h*2)-2)--Why -2 ...?
			else
				SliceCommander:SetSizeBorder(w-h, 0, v['obj'].border)
				v['obj']:SCWidth(w-h)
			end
		end
	end

	VTimerEnergy:SCWidth(w)
	SliceCommander:SetSizeBorder(w, 0, VTimerEnergy.border)
	SliceCommanderFrame:SCWidth(w+16)

	SC.OTHER['AS']['obj']:SCWidth(w)
	SC.BARS['CP']['obj']:SCWidth(w)

	SC.BARS['THREAT']['obj']:SCWidth(w)
	SliceCommander:SetSizeBorder(w, 0, SC.BARS['THREAT']['obj'].border)
	SC.BARS['HEALTH']['obj']:SCWidth(w)
	SliceCommander:SetSizeBorder(w, 0, SC.BARS['HEALTH']['obj'].border)
	SC.BARS['THEALTH']['obj']:SCWidth(w)
	SliceCommander:SetSizeBorder(w, 0, SC.BARS['THEALTH']['obj'].border)

	local cx = 0
	for i = 1, maxCP do
		SC.BARS['CP']['obj'].combos[i]:ClearAllPoints()
		SC.BARS['CP']['obj'].combos[i]:SCPoint("TOPLEFT", SC.BARS['CP']['obj'], "TOPLEFT", cx, 0)
		SC.BARS['CP']['obj'].combos[i]:SCPoint("BOTTOMRIGHT", SC.BARS['CP']['obj'], "BOTTOMLEFT", cx + widthCP , 0)
		SC.BARS['CP']['obj'].combos[i]:SCWidth(widthCP)
		SliceCommander:SetSizeBorder(widthCP, 0, SC.BARS['CP']['obj'].combos[i].border)
		cx = cx + widthCP + pxMargin
	end
	SliceCommander:SetSparks()
end

function SliceCommander:SetSizeBorder(w, h, border)
	local k = 6
	for i,v in pairs(border) do
		if h ~= 0 then
			v:SCHeight(h+k)
		end
		if w ~= 0 then
			v:SCWidth(w+k)
		end
		k = k-2
	end
end

function SliceCommander:SetEnergyHeight(h)
	VTimerEnergy:SCHeight(h)
	SliceCommanderFrame:SCHeight(h+16)
	VTimerEnergyTxt:SCHeight(h)
	SliceCommander_Combo:SCHeight(h)
	SC.Spark[1]:SCHeight(h)
	SC.Spark[2]:SCHeight(h)
	SC.Spark[3]:SCHeight(h)
	SliceCommander:SetSizeBorder(0, h, VTimerEnergy.border)
	SliceCommander:HealthBar('HEALTH')
	SliceCommander:HealthBar('THEALTH')
end

function SliceCommander:SetIconSize(w)
	SliceCommander:SetIconSizeObj(SC.OTHER['POISON']['obj'], w)
	SliceCommander:SetIconSizeObj(SC.OTHER['BF']['obj'], w)
	SliceCommander:SetIconSizeObj(SC.OTHER['SOTD']['obj'], w)
end

function SliceCommander:SetIconSizeObj(obj, w)
	obj:SCSize(w, w)
	obj.overlay:SCSize(w, w)
	obj.iconTxt:SCSize(w, w-5)
	obj.icon:SCSize(w, w)
	obj.moveframe:SCSize(w+16, w+16)
	SliceCommander:SetSizeBorder(w, w, obj.border)
end

function SliceCommander:SetTimersHeight(h)
	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil and i ~= 'CP' then
			v['obj']:SCHeight(h)
			v['obj'].text:SCHeight(h)
			v['obj'].text2:SCHeight(h)

			SliceCommander:SetSizeBorder(0, h, v['obj'].border)
			if i ~= 'THREAT' and i ~= 'HEALTH' and i ~= 'THEALTH' then
				v['obj'].icon:SCSize(h, h)
				SliceCommander:SetSizeBorder(h, h, v['obj'].iconBorder)
			end

			if i == 'RUP' then
				v['dot']['obj']:SCSize(h, h)
				SliceCommander:SetSizeBorder(h, h, v['dot']['obj'].border)
			end
		end
	end

	if _G['SCtick'] == nil then _G['SCtick'] = {} end
	for k, v in pairs(_G['SCtick']) do
		v:SCHeight(h)
	end

	SC.BARS['CP']['obj']:SCHeight(h)
	SC.BARS['CP']['obj'].comboText:SCHeight(h)
	local maxCP = UnitPowerMax("player", SPELL_POWER_COMBO_POINTS)
	for i = 1, maxCP do
		SC.BARS['CP']['obj'].combos[i]:SCHeight(h)
		SliceCommander:SetSizeBorder(0, h, SC.BARS['CP']['obj'].combos[i].border)
	end

	SC.BarFont1:SetFont("Interface\\AddOns\\SliceCommander\\Fonts\\FRIZQT__.TTF", (h-2)*SC.V.pxH)
	SC.BarFont2:SetFont("Interface\\AddOns\\SliceCommander\\Fonts\\FRIZQT__.TTF", (h-2)*SC.V.pxH)
	SC.BARS['CP']['obj'].comboText:SetFont("Interface\\AddOns\\SliceCommander\\Fonts\\FRIZQT__.TTF", (h-2)*SC.V.pxH, "THINOUTLINE")
	SliceCommander:SetWidth(SC.SET.Width)
end

function SliceCommander:FrameOpacity(opValue)
	SliceCommander:SetTransparency(opValue)

	for i,v in pairs(SC.BARS) do
		if v['expire'] ~= nil then
			v['obj']:SetAlpha(opValue/100)
		end
	end

	SC.OTHER['BF']['obj']:SetAlpha(opValue/100)
end

function SliceCommander:TestCurrentVersion()
	if _G['SC'].version == nil or _G['SC'].version < '7.00' then
		_G['SC'] = {version = GetAddOnMetadata("SliceCommander", "Version")}
	else
		_G['SC'].version = GetAddOnMetadata("SliceCommander", "Version")
	end
end

function SliceCommander:DisplaySpells(Spells)
	for i, v in ipairs(Spells) do
		if SC.SET.display[v] == nil then
			SC.SET.display[v] = true
		end
	end
end

function SliceCommander:NotDisplaySpells(Spells)
	for i, v in ipairs(Spells) do
		if SC.SET.display[v] == nil then
			SC.SET.display[v] = false
		end
	end
end

function SliceCommander:OrderSpells(Spells)
	for i, v in pairs(Spells) do
		_G['SC'].orderBar = _G['SC'].orderBar+1
		if _G['SC'].orderBar > _G['SC'].maxPos then
			_G['SC'].maxPos = _G['SC'].orderBar
		end
		if SC.SET.Pos[v] == nil then
			SC.SET.Pos[v] = _G['SC'].orderBar
		end
	end
end

function SliceCommander:OrderCDs(Spells)
	for i, v in pairs(Spells) do
		_G['SC'].orderCDBar = _G['SC'].orderCDBar-1
		if _G['SC'].orderCDBar < _G['SC'].maxCDPos then
			_G['SC'].maxCDPos = _G['SC'].orderCDBar
		end
		if SC.SET.Pos[v] == nil then
			SC.SET.Pos[v] = _G['SC'].orderCDBar
		end
	end

end

function SliceCommander:Global()
	SliceCommander:DisplaySpells({'CP', 'THREAT', 'CS', 'KS', 'KSCD', 'FOTD', 'CV', 'CVCD', 'TOT', 'FE', 'SS', 'SUB', 'CD', 'AC', 'MD', 'DFA', 'Poison', 'SOTD' })
	SliceCommander:NotDisplaySpells({'HEALTH', 'THEALTH', 'VAN'})

	_G['SC'].orderBar = _G['SC'].maxPos
	_G['SC'].orderCDBar = _G['SC'].maxCDPos
	
	if SC.SET.Pos.CP == nil then--Combo Points
		SC.SET.Pos.CP = -1
	end
	
	SliceCommander:OrderSpells({'FOTD', 'CV', 'FE', 'SUB', 'AC'})
	SliceCommander:OrderCDs({'CVCD', 'TOT', 'VAN', 'SS', 'CD', 'MD', 'DFA', 'THREAT', 'HEALTH', 'THEALTH', 'CS', 'KS', 'KSCD'})
end

function SliceCommander:Assassination()
	SliceCommander:DisplaySpells({'RUP', 'GAR', 'GARCD', 'VEN', 'VENCD', 'ENV', 'HEM', 'EP', 'DPICON', 'DPBUT', 'EX', 'KB', 'KBCD', 'BA', 'ST'})
	SliceCommander:NotDisplaySpells({'DP'})

	_G['SC'].orderBar = 0
	_G['SC'].orderCDBar = -1
	
	SliceCommander:OrderSpells({'GAR', 'RUP', 'ENV', 'KB', 'BA', 'ST', 'HEM', 'VEN', 'DP', 'EP'})
	SliceCommander:OrderCDs({'VENCD', 'KBCD', 'EX', 'GARCD'})
end

function SliceCommander:Outlaw()
	SliceCommander:DisplaySpells({'SND', 'RB', 'AR', 'ARCD', 'KSP', 'GS', 'GH', 'CB', 'BE', 'BF', 'OP'})
	SliceCommander:NotDisplaySpells({'PA'})

	_G['SC'].orderBar = 0
	_G['SC'].orderCDBar = -1
	
	SliceCommander:OrderSpells({'SND', 'RB', 'AR', 'GS', 'OP'})
	SliceCommander:OrderCDs({'BE', 'ARCD', 'KSP', 'CB', 'GH', 'PA'})
end

function SliceCommander:Subtlety()
	SliceCommander:DisplaySpells({'SD', 'SDCD', 'SB', 'SBCD', 'ES', 'NB', 'SOD', 'GB' })

	_G['SC'].orderBar = 0
	_G['SC'].orderCDBar = -1
	
	SliceCommander:OrderSpells({'ES', 'NB', 'SOD', 'SD', 'SB'})
	SliceCommander:OrderCDs({'GB', 'SDCD', 'SBCD'})
	
end

function SliceCommander:DisplayOrderBar()
	SliceCommander:Assassination()
	SliceCommander:Outlaw()
	SliceCommander:Subtlety()
	SliceCommander:Global()
end

function SliceCommander:SetSetOptions()
	local defaultTable = {'power', 'poison'}
	for k, v in pairs(defaultTable) do
		if SC.SET[v] == nil then
			SC.SET[v] = {}
		end
	end
	local defaultEnabled = {'PadLatency', 'SpellText', 'SoundCombat', 'ThreatGroupOnly', 'kickEnable', 'displayGCD'}
	for k, v in pairs(defaultEnabled) do
		if SC.SET[v] == nil then
			SC.SET[v] = true
		end
	end
	local defaultDisabled = {'IsLocked', 'HideEnergy', 'FullTimer', 'ARMarioMod', 'skin'}
	for k, v in pairs(defaultDisabled) do
		if SC.SET[v] == nil then
			SC.SET[v] = false
		end
	end
end

function SliceCommander:SetRB()
	if SC.SET.RB == nil then
		SC.SET.RB = {
			['JR'] = 'SaS',
			['GM'] = 'AS',
			['SIW'] = 'Crit',
			['TB'] = 'CDR',
			['BT'] = 'NRJ',
			['BS'] = 'CP+'
		}
		SC.SET.display.RBName = true
		SC.BARS['RB'].subbar = {
			'JR',
			'GM',
			'SIW',
			'TB',
			'BT',
			'BS'
		}
	end
end

function SliceCommander:TestValueOptions()
	if SC.SET == nil or SC.SET.display == nil or SC.SET.Pos == nil then
		SC.SET = { display = {}, Pos = {} }
	end
	if _G['SC'].SET.PixelPerfect == nil then
		_G['SC'].SET.PixelPerfect = true
	end
	SliceCommander:DisplayOrderBar()
	SliceCommander:SetSetOptions()
	SliceCommander:SetRB()
	if SC.SET.Energy == nil then
		SC.SET.Energy = {
			[1] = {--ASSASSINATION
				[1] = 25,
				[2] = 35,
				[3] = 55,
			},
			[2] = {--OUTLAW
				[1] = 25,
				[2] = 35,
				[3] = 50,
			},
			[3] = {--SUBTLETY
				[1] = 25,
				[2] = 35,
				[3] = 40,
			}
		}
	end
	if SC.SET.kickMessage == nil then
		SC.SET.kickMessage = "Kick successful"
	end
	if SC.SET.kickChannel == nil then
		SC.SET.kickChannel = {
			['INSTANCE_CHAT'] = true,
			['RAID'] = true,
			['PARTY'] = true,
			['SAY'] = true
		}
	end
	if SC.SET.power.RUP == nil then
		SC.SET.power.RUP = true
	end
	if SC.SET.eyefinity == nil then
		SC.SET.eyefinity = 1
	end
	if SC.SET.HEALTH == nil then
		SC.SET.HEALTH = {
			value = false,
			HideCombat = true,
			MidUnder = 'None',
			LowUnder = 'None'
		}
	end
	if SC.SET.THEALTH == nil then
		SC.SET.THEALTH = {
			value = false,
			HideCombat = true,
			MidUnder = 'None',
			LowUnder = 'None'
		}
	end
	--[[if SC.SET.EnergySound1 == nil then
		SC.SET.EnergySound1 = 'None'
	end
	if SC.SET.EnergySound2 == nil then
		SC.SET.EnergySound2 = 'None'
	end
	if SC.SET.EnergySound3 == nil then
		SC.SET.EnergySound3 = 'None'
	end]]
	if SC.SET.DPSound == nil then
		SC.SET.DPSound = 'None'
	end
	if SC.SET.FOTDSound == nil then
		SC.SET.FOTDSound = 'OH YEAH'
	end
	if SC.SET.Width == nil then
		SC.SET.Width = 287
	end
	if SC.SET.EnergyTrans == nil then
		SC.SET.EnergyTrans = 0
	end
	if SC.SET.BarMargin == nil then
		SC.SET.BarMargin = 2
	end
	if SC.SET.EnergyHeight == nil then
		SC.SET.EnergyHeight = 28
	end
	if SC.SET.IconSize == nil then
		SC.SET.IconSize = 20
	end
	if SC.SET.TimerHeight == nil then
		SC.SET.TimerHeight = 17
	end
	if SC.SET.BarTexture == nil then
		SC.SET.BarTexture = 'Tukui'
	end
	if SC.SET.display.DPTIME == nil then
		SC.SET.display.DPTIME = 10
	end
	if SC.SET.FrameOpacity == nil then
		SC.SET.FrameOpacity = 100
	end
	if SC.SET.soundChannel == nil then
		SC.SET.soundChannel = "Master"
	end
	if SC.SET.CP == nil then
		SC.SET.CP = {
			[5] = {
				[1] = {.33, .63, .33},
				[2] = {.46, .63, .35},
				[3] = {.65, .63, .35},
				[4] = {.65, .42, .31},
				[5] = {.69, .31, .31},
			},
			[6] = {
				[1] = {.33, .63, .33},
				[2] = {.46, .63, .35},
				[3] = {.65, .63, .35},
				[4] = {.65, .42, .31},
				[5] = {.69, .31, .31},
				[6] = {.74, .20, .27},
			},
			[8] = {
				[1] = {.33, .63, .33},
				[2] = {.46, .63, .35},
				[3] = {.65, .63, .35},
				[4] = {.65, .42, .31},
				[5] = {.69, .31, .31},
				[6] = {.5, .5, .5},
				[7] = {.5, .5, .5},
				[8] = {.5, .5, .5},
			}
		}
	end
	if SC.SET.energyColor == nil then
		SC.SET.energyColor = {.92, .92, .92}
	end
	if SC.SET.poison.mh == nil then
		SC.SET.poison.mh = SC_SPELL['DP'] 
	end
	if SC.SET.poison.oh == nil then
		SC.SET.poison.oh = SC_SPELL['LP'] 
	end
	SliceCommander:InitBarArray()
	SliceCommander:InitOtherArray()
end


function SliceCommander:InitOtherArray()
	if SC.OTHER == nil then
		SC.OTHER = {}
	end
	if SC.OTHER['BF'] == nil or SC.OTHER['BF']['position'] == nil then
		SC.OTHER['BF'] = {
			['obj'] = 0,
			['position'] = {
				['xOfs'] = 45,
				['yOfs'] = 0,
				['relativePoint'] = "CENTER",
				['point'] = "CENTER",
				['relativeTo'] = "UIParent"
			}
		}
	end
	if SC.OTHER['SOTD'] == nil or SC.OTHER['SOTD']['position']== nil then
		SC.OTHER['SOTD'] = {
			['obj'] = 0,
			['iconTxt'] = 0,
			['overlay'] = false,
			['position'] = {
				['xOfs'] = 30,
				['yOfs'] = 0,
				['relativePoint'] = "CENTER",
				['point'] = "CENTER",
				['relativeTo'] = "UIParent"
			}
		}
	end
	if SC.OTHER['AS'] == nil then
		SC.OTHER['AS'] = {
			['obj'] = 0,
			['expire'] = 0,
			['color'] = 0
		}
	end
	if SC.OTHER['POISON'] == nil then
		SC.OTHER['POISON'] = {
			['obj'] = 0,
			['expire'] = 0,
			['color'] = 0,
			['position'] = {
				['xOfs'] = 0,
				['yOfs'] = 0,
				['relativePoint'] = "CENTER",
				['point'] = "CENTER",
				['relativeTo'] = "UIParent"
			}
		}
	end
	if SC.OTHER['SliceCommanderFrame'] == nil then
		SC.OTHER['SliceCommanderFrame'] = {
			['position'] = {
				['xOfs'] = 0,
				['yOfs'] = 0,
				['relativePoint'] = "CENTER",
				['point'] = "CENTER",
				['relativeTo'] = "UIParent"
			}
		}
	end
end

function SliceCommander:InitGlobalBarArray()
	if SC.BARS['HEALTH'] == nil then
		SC.BARS['HEALTH'] = {['color'] = {0, 0, 0}}
	end
	SC.BARS['HEALTH']['target'] = "player"
	if SC.BARS['THEALTH'] == nil then
		SC.BARS['THEALTH'] = {['color'] = {0, 0, 0}}
	end
	SC.BARS['THEALTH']['target'] = "target"
	if SC.BARS['THREAT'] == nil then
		SC.BARS['THREAT'] = {['color'] = {0, 0, 0}}
	end
	SC.BARS['CP'] = {
		['last'] = 0,--last CP has to be init to 0
		['target'] = '',
		['color'] = 0
	}
	
	if SC.BARS['KS'] == nil then--Kidney Shot
		SC.BARS['KS'] = {['color'] = {.89, .73, .52}}
	end
	if SC.BARS['KSCD'] == nil then--Kidney Shot CD
		SC.BARS['KSCD'] = {['color'] = {.89, .73, .52}}
	end
	if SC.BARS['CS'] == nil then--Cheap Shot
		SC.BARS['CS'] = {['color'] = {.89, .73, .2}}
	end
	if SC.BARS['VAN'] == nil then--Vanish
		SC.BARS['VAN'] = {['color'] = {.42, .46, .51}}
	end
	if SC.BARS['FOTD'] == nil then--Fury of the Destroyer
		SC.BARS['FOTD'] = {['color'] = {.7, .13, .75}}
	end
	if SC.BARS['SS'] == nil then--Shadowstep
		SC.BARS['SS'] = {['color'] = {.76, .28, .78}}
	end
	if SC.BARS['FE'] == nil then--Feint
		SC.BARS['FE'] = {['color'] = {.15, .42, .68}}
	end
	if SC.BARS['MD'] == nil then--Marked for Death
		SC.BARS['MD'] = {['color'] = {.92, .33, .33}}
	end
	if SC.BARS['DFA'] == nil then--Death from Above
		SC.BARS['DFA'] = {['color'] = {.75, .65, .53}}
	end
	if SC.BARS['AC'] == nil then--Alacrity
		SC.BARS['AC'] = {['color'] = {.6, .55, .17}}
	end
	if SC.BARS['CV'] == nil then--Crimson Vial
		SC.BARS['CV'] = {['color'] = {.43, .67, .64}}
	end
	SC.BARS['CV']['tick'] = {
		['obj'] = {},
		['value'] = 1
	}
	if SC.BARS['CVCD'] == nil then--Crimson Vial
		SC.BARS['CVCD'] = {['color'] = {.33, .57, .54}}
	end
	if SC.BARS['CD'] == nil then--Cheated Death
		SC.BARS['CD'] = {['color'] = {.2, .47, .19}}
	end
	if SC.BARS['SUB'] == nil then--Subterfuge
		SC.BARS['SUB'] = {['color'] = {.04, .81, .82}}
	end
	if SC.BARS['TOT'] == nil then--Tricks of the Trade
		SC.BARS['TOT'] = {['color'] = {.3, .3, 1}}
	end
end

function SliceCommander:InitAssassinationBarArray()
	if SC.BARS['GAR'] == nil then--Garrote
		SC.BARS['GAR'] = {['color'] = {.83, .27, .27}}
	end
	SC.BARS['GAR']['tick'] = {
		['obj'] = {},
		['value'] = 3
	}
	if SC.BARS['GARCD'] == nil then--Garrote CD
		SC.BARS['GARCD'] = {['color'] = {.83, .27, .27}}
	end
	if SC.BARS['RUP'] == nil then--Rupture
		SC.BARS['RUP'] = {['color'] = {.83, .27, .27}}
	end
	SC.BARS['RUP']['tick'] = {
		['obj'] = {},
		['value'] = 2
	}
	SC.BARS['RUP']['dot'] = {
		['obj'] = 0,
		['power'] = 0
	}
	if SC.BARS['BA'] == nil then--Blood of the Assassinated
		SC.BARS['BA'] = {['color'] = {.83, .27, .27}}
	end
	if SC.BARS['EX'] == nil then--Exsanguinate
		SC.BARS['EX'] = {['color'] = {.83, .27, .27}}
	end
	if SC.BARS['HEM'] == nil then--Hemorrhage
		SC.BARS['HEM'] = {['color'] = {.66, .22, .47}}
	end
	if SC.BARS['DP'] == nil then--Deadly Poison
		SC.BARS['DP'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['ENV'] == nil then--Envenom
		SC.BARS['ENV'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['ST'] == nil then--Surge of Toxins
		SC.BARS['ST'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['KB'] == nil then--Kingsbane
		SC.BARS['KB'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['KBCD'] == nil then--Kingsbane CD
		SC.BARS['KBCD'] = {['color'] = {.27, .68, .23}}
	end
	if SC.BARS['EP'] == nil then--Elaborate Planning
		SC.BARS['EP'] = {['color'] = {.83, .78, .27}}
	end
	if SC.BARS['VEN'] == nil then--Vendetta
		SC.BARS['VEN'] = {['color'] = {.83, .53, .27}}
	end
	if SC.BARS['VENCD'] == nil then--Vendetta CD
		SC.BARS['VENCD'] = {['color'] = {.71, .4, .15}}
	end
end

function SliceCommander:InitOutlawBarArray()--http://paletton.com/#uid=60s0u0klqqvbNEzh9uFpomItmi0
	if SC.BARS['SND'] == nil then--Slice and Dice
		SC.BARS['SND'] = {['color'] = {.83, .51, .27}}
	end
	if SC.BARS['RB'] == nil then--Roll the Bones
		SC.BARS['RB'] = {['color'] = {.83, .51, .27}}
	end
	if SC.BARS['AR'] == nil then--Adrenaline Rush
		SC.BARS['AR'] = {['color'] = {.83, .64, .27}}
	end
	if SC.BARS['ARCD'] == nil then--Adrenaline Rush CD
		SC.BARS['ARCD'] = {['color'] = {.71, .52, .15}}
	end
	if SC.BARS['GS'] == nil then--Ghostly Strike
		SC.BARS['GS'] = {['color'] = {.81, .27, .3}}
	end
	if SC.BARS['CB'] == nil then--Cannonball Barrage
	SC.BARS['CB'] = {['color'] = {.65, .22, .48}}
	end
	if SC.BARS['OP'] == nil then--Opportunity
		SC.BARS['OP'] = {['color'] = {.83, .73, .27}}
	end
	if SC.BARS['PA'] == nil then--Parley
		SC.BARS['PA'] = {['color'] = {.83, .73, .27}}
	end
	if SC.BARS['GH'] == nil then--Grappling Hook
		SC.BARS['GH'] = {['color'] = {.83, .82, .27}}
	end
	if SC.BARS['BE'] == nil then--Between the Eyes
		SC.BARS['BE'] = {['color'] = {.62, .78, .26}}
	end
	if SC.BARS['KSP'] == nil then--Killing Spree
		SC.BARS['KSP'] = {['color'] = {.27, .68, .23}}
	end
end

function SliceCommander:InitSubtletyBarArray()--http://paletton.com/#uid=64r0u0kleqtbzEKgVuIpcmGtdhZ
	if SC.BARS['GB'] == nil then--Goremaw's Bite
	SC.BARS['GB'] = {['color'] = {.52, .18, .54}}
	end
	if SC.BARS['ES'] == nil then--Enveloping Shadows
		SC.BARS['ES'] = {['color'] = {.36, .22, .56}}
	end
	SC.BARS['ES']['tick'] = {
		['obj'] = {},
		['value'] = 3
	}
	if SC.BARS['NB'] == nil then--Nightblade
		SC.BARS['NB'] = {['color'] = {.52, .18, .54}}
	end
	if SC.BARS['SOD'] == nil then--Symbols of Death
		SC.BARS['SOD'] = {['color'] = {.25, .26, .58}}
	end
	SC.BARS['NB']['tick'] = {
		['obj'] = {},
		['value'] = 2
	}
	if SC.BARS['SB'] == nil then--Shadow Blades
		SC.BARS['SB'] = {['color'] = {.19, .58, .39}}
	end
	if SC.BARS['SBCD'] == nil then--Shadow Blades CD
		SC.BARS['SBCD'] = {['color'] = {.1, .5, .31}}
	end
	if SC.BARS['SD'] == nil then--Shadow Dance
		SC.BARS['SD'] = {['color'] = {.2, .4, .53}}
	end
	if SC.BARS['SDCD'] == nil then--Shadow Dance CD
		SC.BARS['SDCD'] = {['color'] = {.12, .32, .45}}
	end
end

function SliceCommander:InitBarArray()
	if SC.BARS == nil then
		SC.BARS = {}
	end
	
	SliceCommander:InitGlobalBarArray()
	SliceCommander:InitAssassinationBarArray()
	SliceCommander:InitOutlawBarArray()
	SliceCommander:InitSubtletyBarArray()
	
	for k, v in pairs(SC.BARS) do
		SC.BARS[k]['obj'] = 0
		SC.BARS[k]['expire'] = 0
		SC.BARS[k]['name'] = SC_SPELL[k]
	end

	local buff = {'FOTD', 'AR', 'SUB', 'EP', 'CV', 'AC', 'DFA', 'FE', 'SND', 'SD', 'SOD', 'ES', 'SB', 'OP', 'ENV'}
	for i, v in ipairs(buff) do
		SC.BARS[v]['type'] = 1
	end
	
	local debuff = {'GAR', 'HEM', 'RUP', 'DP', 'KS', 'CS', 'NB', 'VEN', 'GS', 'KB', 'BA', 'ST'}
	for i, v in ipairs(debuff) do
		SC.BARS[v]['type'] = 2
	end
	
	local selfdebuff = {'CD'}
	for i, v in ipairs(selfdebuff) do
		SC.BARS[v]['type'] = 3
	end
	
	local cd = {'SS', 'VAN', 'KSP', 'ARCD', 'CVCD', 'MD', 'GH', 'PA', 'CB', 'BE', 'TOT', 'SDCD', 'SBCD', 'VENCD', 'GB', 'EX', 'GARCD', 'KBCD', 'KSCD'}
	for i, v in ipairs(cd) do
		SC.BARS[v]['type'] = 4
	end
	
	local chargecd = {'SDCD'}
	for i, v in ipairs(chargecd) do
		SC.BARS[v]['type'] = 5
	end
	
	local chargecd = {'RB'}
	for i, v in ipairs(chargecd) do
		SC.BARS[v]['type'] = 6
	end
	
	SliceCommander:InitSound()
end

function SliceCommander:InitSound()
	if SC.BARS['SND']['sound'] == nil then
		SC.BARS['SND']['sound'] = {
			['alert3'] = 'None',
			['alert2'] = 'None',
			['alert1'] = 'None',
			['refresh3'] = 'None',
			['refresh2'] = 'None',
			['refresh1'] = 'None',
			['applied'] = 'None',
			['expired'] = 'None',
		}
	end
	if SC.BARS['RUP']['sound'] == nil then
		SC.BARS['RUP']['sound'] = {
			['alert3'] = 'None',
			['alert2'] = 'None',
			['alert1'] = 'None',
			['refresh3'] = 'None',
			['refresh2'] = 'None',
			['refresh1'] = 'None',
			['applied'] = 'None',
			['expired'] = 'None',
		}
	end
	if SC.BARS['AR']['sound'] == nil then
		SC.SET.ARMarioMod = 0
		SC.BARS['AR']['sound'] = {
			['applied'] = 'None',
			["alert1"] = "None",
			["refresh3"] = "None",
			["refresh2"] = "None",
			["expired"] = "None",
			["alert3"] = "None",
			["alert2"] = "None",
			["refresh1"] = "None"
		}
	end
end